@@ -1,4 +1,5 @@
 ;;; ielm.el --- interaction mode for Emacs Lisp
+
 ;; Copyright (C) 1994 Free Software Foundation, Inc.
 
 ;; Author: David Smith <maa036@lancaster.ac.uk>
@@ -18,8 +19,9 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs; see the file COPYING.  If not, write to
-;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+;; along with GNU Emacs; see the file COPYING.  If not, write to the
+;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+;; Boston, MA 02111-1307, USA.
 
 ;;; Commentary:
 
@@ -87,7 +89,7 @@ This variable is buffer-local.")
 (defvar ielm-header 
   (concat
    "*** Welcome to IELM version "
-   (substring "$Revision: 1.4 $" 11 -2)
+   (substring "$Revision: 1.7 $" 11 -2)
    " ***  Type (describe-mode) for help.\n"
    "IELM has ABSOLUTELY NO WARRANTY; type (describe-no-warranty) for details.\n")
   "Message to display when IELM is started.")
@@ -116,6 +118,12 @@ This variable is buffer-local.")
   (define-key ielm-map "\C-c\C-f" 'ielm-display-working-buffer)
   (define-key ielm-map "\C-c\C-v" 'ielm-print-working-buffer))
 
+(defvar ielm-font-lock-keywords
+  (list 
+   (cons (concat "^" (regexp-quote ielm-prompt)) 'font-lock-keyword-face)
+   '("\\(^\\*\\*\\*[^*]+\\*\\*\\*\\)\\(.*$\\)" (1 font-lock-comment-face) (2 font-lock-reference-face)))
+  "Additional expressions to highlight in ielm buffers.")
+	
 ;;; Completion stuff
 
 (defun ielm-tab nil
@@ -139,7 +147,7 @@ This variable is buffer-local.")
     (or 
      ;; the buffer has been modified
      (not (= btick (buffer-modified-tick))) 
-     ;; a completions buffer has been modifed or created
+     ;; a completions buffer has been modified or created
      (if cbuffer
 	 (not (= ctick (buffer-modified-tick cbuffer)))
        (get-buffer "*Completions*")))))
@@ -417,6 +425,11 @@ Customised bindings may be defined in `ielm-map', which currently contains:
   (setq ::: nil)
   (make-local-variable ':::)
 
+  ;; font-lock support
+  (make-local-variable 'font-lock-defaults)
+  (setq font-lock-defaults 
+	'(ielm-font-lock-keywords nil nil ((?: . "w") (?- . "w") (?* . "w"))))
+  
   ;; A dummy process to keep comint happy. It will never get any input
   (if (comint-check-proc (current-buffer)) nil
     (start-process "ielm" (current-buffer) "cat")
