@@ -1,10 +1,9 @@
 ;;; isearch.el --- incremental search minor mode.
 
-;; Copyright (C) 1992, 1993, 1994 Free Software Foundation, Inc.
+;; Copyright (C) 1992, 1993, 1994, 1995 Free Software Foundation, Inc.
 
 ;; Author: Daniel LaLiberte <liberte@cs.uiuc.edu>
-
-;; |$Date: 1994/05/14 09:50:26 $|$Revision: 1.69 $
+;; Maintainer: FSF
 
 ;; This file is part of GNU Emacs.
 
@@ -19,12 +18,12 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs; see the file COPYING.  If not, write to
-;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+;; along with GNU Emacs; see the file COPYING.  If not, write to the
+;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+;; Boston, MA 02111-1307, USA.
 
 ;;; Commentary:
 
-;;;====================================================================
 ;; Instructions
 
 ;; For programmed use of isearch-mode, e.g. calling (isearch-forward),
@@ -66,56 +65,45 @@
 
 ;;; Change Log:
 
-;;; Changes before those recorded in ChangeLog:
+;; Changes before those recorded in ChangeLog:
 
-;;; Revision 1.4  92/09/14  16:26:02  liberte
-;;; Added prefix args to isearch-forward, etc. to switch between
-;;;    string and regular expression searching.
-;;; Added some support for lemacs.
-;;; Added general isearch-highlight option - but only for lemacs so far.
-;;; Added support for frame switching in emacs 19.
-;;; Added word search option to isearch-edit-string.
-;;; Renamed isearch-quit to isearch-abort.
-;;; Numerous changes to comments and doc strings.
-;;; 
-;;; Revision 1.3  92/06/29  13:10:08  liberte
-;;; Moved modal isearch-mode handling into isearch-mode.
-;;; Got rid of buffer-local isearch variables.
-;;; isearch-edit-string used by ring adjustments, completion, and
-;;; nonincremental searching.  C-s and C-r are additional exit commands.
-;;; Renamed all regex to regexp.
-;;; Got rid of found-start and found-point globals.
-;;; Generalized handling of upper-case chars.
- 
-;;; Revision 1.2  92/05/27  11:33:57  liberte
-;;; Emacs version 19 has a search ring, which is supported here.
-;;; Other fixes found in the version 19 isearch are included here.
-;;;
-;;; Also see variables search-caps-disable-folding,
-;;; search-nonincremental-instead, search-whitespace-regexp, and
-;;; commands isearch-toggle-regexp, isearch-edit-string.
-;;;
-;;; semi-modal isearching is supported.
+;; Revision 1.4  92/09/14  16:26:02  liberte
+;; Added prefix args to isearch-forward, etc. to switch between
+;;    string and regular expression searching.
+;; Added some support for lemacs.
+;; Added general isearch-highlight option - but only for lemacs so far.
+;; Added support for frame switching in emacs 19.
+;; Added word search option to isearch-edit-string.
+;; Renamed isearch-quit to isearch-abort.
+;; Numerous changes to comments and doc strings.
+;; 
+;; Revision 1.3  92/06/29  13:10:08  liberte
+;; Moved modal isearch-mode handling into isearch-mode.
+;; Got rid of buffer-local isearch variables.
+;; isearch-edit-string used by ring adjustments, completion, and
+;; nonincremental searching.  C-s and C-r are additional exit commands.
+;; Renamed all regex to regexp.
+;; Got rid of found-start and found-point globals.
+;; Generalized handling of upper-case chars.
 
-;;; Changes for 1.1
-;;; 3/18/92 Fixed invalid-regexp.
-;;; 3/18/92 Fixed yanking in regexps.
+;; Revision 1.2  92/05/27  11:33:57  liberte
+;; Emacs version 19 has a search ring, which is supported here.
+;; Other fixes found in the version 19 isearch are included here.
+;;
+;; Also see variables search-caps-disable-folding,
+;; search-nonincremental-instead, search-whitespace-regexp, and
+;; commands isearch-toggle-regexp, isearch-edit-string.
+;;
+;; semi-modal isearching is supported.
+
+;; Changes for 1.1
+;; 3/18/92 Fixed invalid-regexp.
+;; 3/18/92 Fixed yanking in regexps.
 
 ;;; Code:
 
 
-;;;=========================================================================
-;;; Emacs features
-
-;; isearch-mode takes advantage of the features provided by several
-;; different versions of emacs.  Rather than testing the version of
-;; emacs, several constants are defined, one for each of the features.
-;; Each of the tests below must work on any version of emacs.
-;; (Perhaps provide and featurep could be used for this purpose.)
-
-(defconst isearch-gnu-emacs-events (fboundp 'set-frame-height)) ;; emacs 19
-(defconst isearch-pre-command-hook-exists (boundp 'pre-command-hook)) ;; lemacs
-(defconst isearch-event-data-type nil)  ;; lemacs
+;;; Some additional options and constants.
 
 (defconst search-exit-option t
   "*Non-nil means random control characters terminate incremental search.")
@@ -131,9 +119,6 @@ and the value is minus the number of lines.")
 This is the style where a one-line window is created to show the line
 that the search has reached.")
 
-;;;========================================================================
-;;; Some additional options and constants.
-
 (defvar search-upper-case 'not-yanks
   "*If non-nil, upper case chars disable case fold searching.
 That is, upper and lower case chars must match exactly.
@@ -150,9 +135,6 @@ string, and RET terminates editing and does a nonincremental search.")
   "*If non-nil, regular expression to match a sequence of whitespace chars.
 You might want to use something like \"[ \\t\\r\\n]+\" instead.")
 
-;; I removed the * from the doc string because highlighting is not 
-;; currently a clean thing to do.  Once highlighting is made clean, 
-;; this feature can be re-enabled and advertised.
 (defvar search-highlight nil
   "*Non-nil means incremental search highlights the current match.")
 
@@ -162,7 +144,6 @@ You might want to use something like \"[ \\t\\r\\n]+\" instead.")
 (defvar isearch-mode-end-hook nil
   "Function(s) to call after terminating an incremental search.")
 
-;;;==================================================================
 ;;; Search ring.
 
 (defvar search-ring nil
@@ -186,7 +167,6 @@ nil if none yet.")
   "*Non-nil if advancing or retreating in the search ring should cause search.
 Default value, nil, means edit the string instead.")
 
-;;;====================================================
 ;;; Define isearch-mode keymap.
 
 (defvar isearch-mode-map nil
@@ -217,11 +197,26 @@ Default value, nil, means edit the string instead.")
 	(define-key map (vector i) 'isearch-printing-char)
 	(setq i (1+ i)))
 
+      ;; To handle local bindings with meta char prefix keys, define
+      ;; another full keymap.  This must be done for any other prefix
+      ;; keys as well, one full keymap per char of the prefix key.  It
+      ;; would be simpler to disable the global keymap, and/or have a
+      ;; default local key binding for any key not otherwise bound.
+      (let ((meta-map (make-sparse-keymap)))
+	(define-key map (char-to-string meta-prefix-char) meta-map)
+	(define-key map [escape] meta-map))
+      (define-key map (vector meta-prefix-char t) 'isearch-other-meta-char)
+
       ;; Several non-printing chars change the searching behavior.
       (define-key map "\C-s" 'isearch-repeat-forward)
       (define-key map "\C-r" 'isearch-repeat-backward)
       (define-key map "\177" 'isearch-delete-char)
       (define-key map "\C-g" 'isearch-abort)
+      ;; This assumes \e is the meta-prefix-char.
+      (or (= ?\e meta-prefix-char)
+	  (error "Inconsistency in isearch.el"))
+      (define-key map "\e\e\e" 'isearch-cancel)
+      (define-key map  [escape escape escape] 'isearch-cancel)
     
       (define-key map "\C-q" 'isearch-quote-char)
 
@@ -232,39 +227,6 @@ Default value, nil, means edit the string instead.")
     
       (define-key map "\C-w" 'isearch-yank-word)
       (define-key map "\C-y" 'isearch-yank-line)
-      (define-key map [mouse-2] 'isearch-yank-kill)
-      ;; This overrides the default binding for t.
-      (define-key map [down-mouse-2] 'nil)
-
-      ;; Bind the ASCII-equivalent "function keys" explicitly
-      ;; if we bind their equivalents, 
-      ;; since otherwise the default binding would override.
-      ;; We bind [escape] below.
-      (define-key map [tab] 'isearch-printing-char)
-      (define-key map [kp-0] 'isearch-printing-char)
-      (define-key map [kp-1] 'isearch-printing-char)
-      (define-key map [kp-2] 'isearch-printing-char)
-      (define-key map [kp-3] 'isearch-printing-char)
-      (define-key map [kp-4] 'isearch-printing-char)
-      (define-key map [kp-5] 'isearch-printing-char)
-      (define-key map [kp-6] 'isearch-printing-char)
-      (define-key map [kp-7] 'isearch-printing-char)
-      (define-key map [kp-8] 'isearch-printing-char)
-      (define-key map [kp-9] 'isearch-printing-char)
-      (define-key map [kp-add] 'isearch-printing-char)
-      (define-key map [kp-subtract] 'isearch-printing-char)
-      (define-key map [kp-multiply] 'isearch-printing-char)
-      (define-key map [kp-divide] 'isearch-printing-char)
-      (define-key map [kp-decimal] 'isearch-printing-char)
-      (define-key map [kp-separator] 'isearch-printing-char)
-      (define-key map [kp-equal] 'isearch-printing-char)
-      (define-key map [kp-tab] 'isearch-printing-char)
-      (define-key map [kp-space] 'isearch-printing-char)
-      (define-key map [kp-enter] 'isearch-exit)
-      (define-key map [delete] 'isearch-delete-char)
-      (define-key map [backspace] 'isearch-delete-char)
-      (define-key map [return] 'isearch-exit)
-      (define-key map [newline] 'isearch-printing-char)
 
       ;; Define keys for regexp chars * ? |.
       ;; Nothing special for + because it matches at least once.
@@ -276,26 +238,20 @@ Default value, nil, means edit the string instead.")
 ;;;      ;; Instead bind C-h to special help command for isearch-mode.
 ;;;      (define-key map "\C-h" 'isearch-mode-help)
 
-      ;; To handle local bindings with meta char prefix keys, define
-      ;; another full keymap.  This must be done for any other prefix
-      ;; keys as well, one full keymap per char of the prefix key.  It
-      ;; would be simpler to disable the global keymap, and/or have a
-      ;; default local key binding for any key not otherwise bound.
-      (let ((meta-map (make-sparse-keymap)))
-	(define-key map (char-to-string meta-prefix-char) meta-map)
-	(define-key map [escape] meta-map))
-      (define-key map (vector meta-prefix-char t) 'isearch-other-meta-char)
-
       (define-key map "\M-n" 'isearch-ring-advance)
       (define-key map "\M-p" 'isearch-ring-retreat)
       (define-key map "\M-y" 'isearch-yank-kill)
 
       (define-key map "\M-\t" 'isearch-complete)
 
-      ;; For emacs 19, switching frames should terminate isearch-mode
-      (if isearch-gnu-emacs-events
-	  (define-key map [switch-frame] 'isearch-switch-frame-handler))
-      
+      ;; Pass frame events transparently so they won't exit the search.
+      ;; In particular, if we have more than one display open, then a
+      ;; switch-frame might be generated by someone typing at another keyboard.
+      (define-key map [switch-frame] nil)
+      (define-key map [delete-frame] nil)
+      (define-key map [iconify-frame] nil)
+      (define-key map [make-frame-visible] nil)
+
       (setq isearch-mode-map map)
       ))
 
@@ -320,7 +276,6 @@ Default value, nil, means edit the string instead.")
       (setq minibuffer-local-isearch-map map)
       ))
 
-;;;========================================================
 ;; Internal variables declared globally for byte-compiler.
 ;; These are all set with setq while isearching
 ;; and bound locally while editing the search string.
@@ -339,6 +294,7 @@ Default value, nil, means edit the string instead.")
 (defvar isearch-other-end nil)	; Start (end) of match if forward (backward).
 (defvar isearch-wrapped nil)	; Searching restarted from the top (bottom).
 (defvar isearch-barrier 0)
+(defvar isearch-just-started nil)
 
 ; case-fold-search while searching.
 ;   either nil, t, or 'yes.  'yes means the same as t except that mixed
@@ -371,7 +327,6 @@ Default value, nil, means edit the string instead.")
 (defvar isearch-new-forward nil)
 
 
-;;;==============================================================
 ;; Minor-mode-alist changes - kind of redundant with the
 ;; echo area, but if isearching in multiple windows, it can be useful.
 
@@ -387,7 +342,6 @@ Default value, nil, means edit the string instead.")
 (define-key global-map "\C-r" 'isearch-backward)
 (define-key esc-map "\C-r" 'isearch-backward-regexp)
 
-;;;===============================================================
 ;;; Entry points to isearch-mode.
 ;;; These four functions should replace those in loaddefs.el
 ;;; An alternative is to defalias isearch-forward etc to isearch-mode,
@@ -427,6 +381,7 @@ The above keys, bound in `isearch-mode-map', are often controlled by
  options; do M-x apropos on search-.* to find them.
 Other control and meta characters terminate the search
  and are then executed normally (depending on `search-exit-option').
+Likewise for function keys and mouse button events.
 
 If this function is called non-interactively, it does not return to
 the calling function until the search is done."
@@ -467,7 +422,6 @@ is treated as a regexp.  See \\[isearch-forward] for more info."
   (isearch-update))
 
 
-;;;==================================================================
 ;; isearch-mode only sets up incremental search for the minor mode.
 ;; All the work is done by the isearch-mode commands.
 
@@ -478,7 +432,9 @@ is treated as a regexp.  See \\[isearch-forward] for more info."
 			     
 
 (defun isearch-mode (forward &optional regexp op-fun recursive-edit word-p)
-  "Start isearch minor mode.  Called by isearch-forward, etc."
+  "Start isearch minor mode.  Called by `isearch-forward', etc.
+
+\\{isearch-mode-map}"
 
   ;; Initialize global vars.
   (setq isearch-forward forward
@@ -496,56 +452,45 @@ is treated as a regexp.  See \\[isearch-forward] for more info."
 	isearch-yank-flag nil
 	isearch-invalid-regexp nil
 	isearch-within-brackets nil
-	;; Use (baud-rate) for now, for sake of other versions.
-	isearch-slow-terminal-mode (and (<= (baud-rate) search-slow-speed)
+	isearch-slow-terminal-mode (and (<= baud-rate search-slow-speed)
 					(> (window-height)
 					   (* 4 search-slow-window-lines)))
 	isearch-other-end nil
 	isearch-small-window nil
+	isearch-just-started t
 
 	isearch-opoint (point)
 	search-ring-yank-pointer nil
 	regexp-search-ring-yank-pointer nil)
+  (looking-at "")
   (setq isearch-window-configuration
 	(if isearch-slow-terminal-mode (current-window-configuration) nil))
 
-;; This was for Lucid Emacs.  But now that we have pre-command-hook,
-;; it causes trouble.
-;;  (if isearch-pre-command-hook-exists
-;;      (add-hook 'pre-command-hook 'isearch-pre-command-hook))
   (setq	isearch-mode " Isearch")  ;; forward? regexp?
-  (set-buffer-modified-p (buffer-modified-p)) ; update modeline
-
-  ;; It is ugly to show region highlighting while the search
-  ;; is going on.  And we don't want the mark active at the end either.
-  (setq deactivate-mark t)      
+  (force-mode-line-update)
 
   (isearch-push-state)
 
-  (make-local-variable 'overriding-local-map)
-  (setq overriding-local-map isearch-mode-map)
+  (setq overriding-terminal-local-map isearch-mode-map)
   (isearch-update)
   (run-hooks 'isearch-mode-hook)
 
+  (add-hook 'mouse-leave-buffer-hook 'isearch-done)
+
   ;; isearch-mode can be made modal (in the sense of not returning to 
   ;; the calling function until searching is completed) by entering 
   ;; a recursive-edit and exiting it when done isearching.
   (if recursive-edit
       (let ((isearch-recursive-edit t))
 	(recursive-edit)))
-  )
+  isearch-success)
 
 
-;;;====================================================
 ;; Some high level utilities.  Others below.
 
 (defun isearch-update ()
   ;; Called after each command to update the display.  
-  (if (if isearch-event-data-type
-	  (null unread-command-event)
-	(if isearch-gnu-emacs-events
-	    (null unread-command-events)
-	  (< unread-command-char 0)))
+  (if (null unread-command-events)
       (progn
 	(if (not (input-pending-p))
 	    (isearch-message))
@@ -579,11 +524,11 @@ is treated as a regexp.  See \\[isearch-forward] for more info."
    isearch-yank-flag nil)
   )
 
-
-(defun isearch-done (&optional nopush)
+(defun isearch-done (&optional nopush edit)
+  (remove-hook 'mouse-leave-buffer-hook 'isearch-done)
   ;; Called by all commands that terminate isearch-mode.
   ;; If NOPUSH is non-nil, we don't push the string on the search ring.
-  (setq overriding-local-map nil)
+  (setq overriding-terminal-local-map nil)
   ;; (setq pre-command-hook isearch-old-pre-command-hook) ; for lemacs
   (isearch-dehighlight t)
   (let ((found-start (window-start (selected-window)))
@@ -594,43 +539,46 @@ is treated as a regexp.  See \\[isearch-forward] for more info."
     (if isearch-small-window
 	(goto-char found-point)
       ;; Exiting the save-window-excursion clobbers window-start; restore it.
-      (set-window-start (selected-window) found-start t)))
+      (set-window-start (selected-window) found-start t))
 
     ;; If there was movement, mark the starting position.
     ;; Maybe should test difference between and set mark iff > threshold.
     (if (/= (point) isearch-opoint)
-	(progn
-	  (push-mark isearch-opoint t)
-	  (or executing-macro (> (minibuffer-depth) 0)
-	      (message "Mark saved where search started")))
-      ;; (message "") why is this needed?
-      )
+	(or (and transient-mark-mode mark-active)
+	    (progn
+	      (push-mark isearch-opoint t)
+	      (or executing-kbd-macro (> (minibuffer-depth) 0)
+		  (message "Mark saved where search started"))))))
 
   (setq isearch-mode nil)
-  (set-buffer-modified-p (buffer-modified-p))  ;; update modeline
+  (force-mode-line-update)
 
   (if (and (> (length isearch-string) 0) (not nopush))
       ;; Update the ring data.
-      (if isearch-regexp 
-	  (if (or (null regexp-search-ring)
-		  (not (string= isearch-string (car regexp-search-ring))))
-	      (progn
-		(setq regexp-search-ring
-		      (cons isearch-string regexp-search-ring))
-		(if (> (length regexp-search-ring) regexp-search-ring-max)
-		    (setcdr (nthcdr (1- search-ring-max) regexp-search-ring)
-			    nil))))
-	(if (or (null search-ring)
-		(not (string= isearch-string (car search-ring))))
-	    (progn
-	      (setq search-ring (cons isearch-string search-ring))
-	      (if (> (length search-ring) search-ring-max)
-		  (setcdr (nthcdr (1- search-ring-max) search-ring) nil))))))
+      (isearch-update-ring isearch-string isearch-regexp))
 
   (run-hooks 'isearch-mode-end-hook)
-  (if isearch-recursive-edit (exit-recursive-edit)))
+  (and (not edit) isearch-recursive-edit (exit-recursive-edit)))
+
+(defun isearch-update-ring (string &optional regexp)
+  "Add STRING to the beginning of the search ring.
+REGEXP says which ring to use."
+  (if regexp 
+      (if (or (null regexp-search-ring)
+	      (not (string= string (car regexp-search-ring))))
+	  (progn
+	    (setq regexp-search-ring
+		  (cons string regexp-search-ring))
+	    (if (> (length regexp-search-ring) regexp-search-ring-max)
+		(setcdr (nthcdr (1- search-ring-max) regexp-search-ring)
+			nil))))
+    (if (or (null search-ring)
+	    (not (string= string (car search-ring))))
+	(progn
+	  (setq search-ring (cons string search-ring))
+	  (if (> (length search-ring) search-ring-max)
+	      (setcdr (nthcdr (1- search-ring-max) search-ring) nil))))))
 
-;;;=======================================================
 ;;; Switching buffers should first terminate isearch-mode.
 ;;; This is done quite differently for each variant of emacs.
 ;;; For lemacs, see Exiting in lemacs below
@@ -642,10 +590,7 @@ is treated as a regexp.  See \\[isearch-forward] for more info."
   (isearch-done)
   (handle-switch-frame (car (cdr (isearch-last-command-char)))))
 
-;;;========================================================
-
 
-;;;====================================================
 ;; Commands active while inside of the isearch minor mode.
 
 (defun isearch-exit ()
@@ -703,7 +648,12 @@ If first char entered is \\[isearch-yank-word], then do word search instead."
 	    (isearch-yank-flag isearch-yank-flag)
 	    (isearch-invalid-regexp isearch-invalid-regexp)
 	    (isearch-within-brackets isearch-within-brackets)
-	    (isearch-other-end isearch-other-end)
+;;; Don't bind this.  We want isearch-search, below, to set it.
+;;; And the old value won't matter after that.
+;;;	    (isearch-other-end isearch-other-end)
+;;; Perhaps some of these other variables should be bound for a
+;;; shorter period, ending before the next isearch-search.
+;;; But there doesn't seem to be a real bug, so let's not risk it now.
 	    (isearch-opoint isearch-opoint)
 	    (isearch-slow-terminal-mode isearch-slow-terminal-mode)
 	    (isearch-small-window isearch-small-window)
@@ -716,7 +666,7 @@ If first char entered is \\[isearch-yank-word], then do word search instead."
 	;; This is so that the user can do anything without failure, 
 	;; like switch buffers and start another isearch, and return.
 	(condition-case err
-	    (isearch-done t)
+	    (isearch-done t t)
 	  (exit nil))			; was recursive editing
 
 	(isearch-message) ;; for read-char
@@ -727,7 +677,8 @@ If first char entered is \\[isearch-yank-word], then do word search instead."
 			(read-event)))
 		   ;; Binding minibuffer-history-symbol to nil is a work-around
 		   ;; for some incompatibility with gmhist.
-		   (minibuffer-history-symbol))
+		   (minibuffer-history-symbol)
+		   (message-log-max nil))
 	      ;; If the first character the user types when we prompt them
 	      ;; for a string is the yank-word character, then go into
 	      ;; word-search mode.  Otherwise unread that character and
@@ -743,17 +694,19 @@ If first char entered is \\[isearch-yank-word], then do word search instead."
 	      (setq cursor-in-echo-area nil)
 	      (setq isearch-new-string
 		    (let (junk-ring)
-		      (read-from-minibuffer (isearch-message-prefix)
-					    isearch-string
-					    minibuffer-local-isearch-map nil
-					    'junk-ring))
-		    isearch-new-message (mapconcat 'text-char-description
-						   isearch-new-string "")))
+		      (read-from-minibuffer
+ 		       (isearch-message-prefix nil nil isearch-nonincremental)
+		       isearch-string
+		       minibuffer-local-isearch-map nil
+		       'junk-ring))
+		    isearch-new-message
+		    (mapconcat 'isearch-text-char-description
+			       isearch-new-string "")))
 	  ;; Always resume isearching by restarting it.
 	  (isearch-mode isearch-forward 
 			isearch-regexp 
 			isearch-op-fun 
-			isearch-recursive-edit
+			nil
 			isearch-word)
 
 	  ;; Copy new local values to isearch globals
@@ -764,8 +717,10 @@ If first char entered is \\[isearch-yank-word], then do word search instead."
 
 	;; Empty isearch-string means use default.
 	(if (= 0 (length isearch-string))
-	    (setq isearch-string (car (if isearch-regexp regexp-search-ring
-					search-ring)))
+	    (setq isearch-string (or (car (if isearch-regexp
+					      regexp-search-ring
+					    search-ring))
+				     ""))
 	  ;; This used to set the last search string,
 	  ;; but I think it is not right to do that here.
 	  ;; Only the string actually used should be saved.
@@ -799,11 +754,17 @@ If first char entered is \\[isearch-yank-word], then do word search instead."
   (setq isearch-new-forward nil)
   (exit-minibuffer))
 
+(defun isearch-cancel ()
+  "Terminate the search and go back to the starting point."
+  (interactive)
+  (goto-char isearch-opoint)
+  (isearch-done t)
+  (signal 'quit nil))  ; and pass on quit signal
 
 (defun isearch-abort ()
-  "Abort incremental search mode if searching is successful, signalling quit.
+  "Abort incremental search mode if searching is successful, signaling quit.
 Otherwise, revert to previous successful search and continue searching.
-Use `isearch-exit' to quit without signalling."
+Use `isearch-exit' to quit without signaling."
   (interactive)
 ;;  (ding)  signal instead below, if quitting
   (discard-input)
@@ -811,13 +772,15 @@ Use `isearch-exit' to quit without signalling."
       ;; If search is successful, move back to starting point
       ;; and really do quit.
       (progn (goto-char isearch-opoint)
+	     (setq isearch-success nil)
 	     (isearch-done t)   ; exit isearch
 	     (signal 'quit nil))  ; and pass on quit signal
-    ;; If search is failing, rub out until it is once more successful.
-    (while (not isearch-success) (isearch-pop-state))
+    ;; If search is failing, or has an incomplete regexp,
+    ;; rub out until it is once more successful.
+    (while (or (not isearch-success) isearch-invalid-regexp)
+      (isearch-pop-state))
     (isearch-update)))
 
-
 (defun isearch-repeat (direction)
   ;; Utility for isearch-repeat-forward and -backward.
   (if (eq isearch-forward (eq direction 'forward))
@@ -845,7 +808,8 @@ Use `isearch-exit' to quit without signalling."
 
   (if (equal isearch-string "")
       (setq isearch-success t)
-    (if (and isearch-success (equal (match-end 0) (match-beginning 0)))
+    (if (and isearch-success (equal (match-end 0) (match-beginning 0))
+	     (not isearch-just-started))
 	;; If repeating a search that found
 	;; an empty string, ensure we advance.
 	(if (if isearch-forward (eobp) (bobp))
@@ -883,10 +847,11 @@ Use `isearch-exit' to quit without signalling."
   (interactive)
   (setq isearch-case-fold-search
 	(if isearch-case-fold-search nil 'yes))
-  (message "%s%s [case %ssensitive]"
-	   (isearch-message-prefix)
-	   isearch-message
-	   (if isearch-case-fold-search "in" ""))
+  (let ((message-log-max nil))
+    (message "%s%s [case %ssensitive]"
+	     (isearch-message-prefix nil nil isearch-nonincremental)
+	     isearch-message
+	     (if isearch-case-fold-search "in" "")))
   (setq isearch-adjusted t)
   (sit-for 1)
   (isearch-update))
@@ -965,8 +930,9 @@ If no previous match was done, just beep."
     ;; long as the match does not extend past search origin.
     (if (and (not isearch-forward) (not isearch-adjusted)
 	     (condition-case ()
-		 (looking-at (if isearch-regexp isearch-string
-			       (regexp-quote isearch-string)))
+		 (let ((case-fold-search isearch-case-fold-search))
+		   (looking-at (if isearch-regexp isearch-string
+				 (regexp-quote isearch-string))))
 	       (error nil))
 	       (or isearch-yank-flag
 		   (<= (match-end 0) 
@@ -1033,37 +999,76 @@ But only if `search-exit-option' is non-nil, the default.
 If it is the symbol `edit', the search string is edited in the minibuffer
 and the meta character is unread so that it applies to editing the string."
   (interactive)
-  (cond ((eq search-exit-option 'edit)
-	 (let ((key (this-command-keys)))
-	   (apply 'isearch-unread (listify-key-sequence key)))
-	 (isearch-edit-string))
-	(search-exit-option
-	 (let ((key (this-command-keys))
-	       (index 0)
-	       window)
-	   (apply 'isearch-unread (listify-key-sequence key))
-	   ;; Properly handle scroll-bar and mode-line clicks
-	   ;; for which a dummy prefix event was generated as (aref key 0).
-	   (and (> (length key) 1)
-		(symbolp (aref key 0))
-		(listp (aref key 1))
-		;; These events now have a symbol; they used to have a list.
-		;; Accept either one.  Other events have a number here.
-		(not (numberp (posn-point (event-start (aref key 1)))))
-		(setq index 1))
-	   ;; If we got a mouse click, maybe it was read with the buffer
-	   ;; it was clicked on.  If so, that buffer, not the current one,
-	   ;; is in isearch mode.  So end the search in that buffer.
-	   (if (and (listp (aref key index))
-		    (setq window (posn-window (event-start (aref key index))))
-		    (windowp window))
-	       (save-excursion
-		 (set-buffer (window-buffer window))
-		 (isearch-done))
-	     (isearch-done))))
-	(t;; otherwise nil
-	 (isearch-process-search-string (this-command-keys)
-					(this-command-keys)))))
+  (let* ((key (this-command-keys))
+	 (main-event (aref key 0))
+	 (keylist (listify-key-sequence key)))
+    (cond ((and (= (length key) 1)
+		(let ((lookup (lookup-key function-key-map key)))
+		  (not (or (null lookup) (integerp lookup)
+			   (keymapp lookup)))))
+	   ;; Handle a function key that translates into something else.
+	   ;; If the key has a global definition too,
+	   ;; exit and unread the key itself, so its global definition runs.
+	   ;; Otherwise, unread the translation,
+	   ;; so that the translated key takes effect within isearch.
+	   (cancel-kbd-macro-events)
+	   (if (lookup-key global-map key)
+	       (progn 
+		 (isearch-done)
+		 (apply 'isearch-unread keylist))
+	     (apply 'isearch-unread
+		    (listify-key-sequence (lookup-key function-key-map key)))))
+	  (
+	   ;; Handle an undefined shifted control character
+	   ;; by downshifting it if that makes it defined.
+	   ;; (As read-key-sequence would normally do,
+	   ;; if we didn't have a default definition.)
+	   (let ((mods (event-modifiers main-event)))
+	     (and (integerp main-event)
+		  (memq 'shift mods)
+		  (memq 'control mods)
+		  (lookup-key isearch-mode-map
+			      (let ((copy (copy-sequence key)))
+				(aset copy 0
+				      (- main-event (- ?\C-\S-a ?\C-a)))
+				copy)
+			      nil)))
+	   (setcar keylist (- main-event (- ?\C-\S-a ?\C-a)))
+	   (cancel-kbd-macro-events)
+	   (apply 'isearch-unread keylist))
+	  ((eq search-exit-option 'edit)
+	   (apply 'isearch-unread keylist)
+	   (isearch-edit-string))
+	  (search-exit-option
+	   (let (window)
+	     (cancel-kbd-macro-events)
+	     (apply 'isearch-unread keylist)
+	     ;; Properly handle scroll-bar and mode-line clicks
+	     ;; for which a dummy prefix event was generated as (aref key 0).
+	     (and (> (length key) 1)
+		  (symbolp (aref key 0))
+		  (listp (aref key 1))
+		  (not (numberp (posn-point (event-start (aref key 1)))))
+		  ;; Convert the event back into its raw form,
+		  ;; with the dummy prefix implicit in the mouse event,
+		  ;; so it will get split up once again.
+		  (progn (setq unread-command-events
+			       (cdr unread-command-events))
+			 (setq main-event (car unread-command-events))
+			 (setcar (cdr (event-start main-event))
+				 (car (nth 1 (event-start main-event))))))
+	     ;; If we got a mouse click, maybe it was read with the buffer
+	     ;; it was clicked on.  If so, that buffer, not the current one,
+	     ;; is in isearch mode.  So end the search in that buffer.
+	     (if (and (listp main-event)
+		      (setq window (posn-window (event-start main-event)))
+		      (windowp window))
+		 (save-excursion
+		   (set-buffer (window-buffer window))
+		   (isearch-done))
+	       (isearch-done))))
+	  (t;; otherwise nil
+	   (isearch-process-search-string key key)))))
 
 (defun isearch-quote-char ()
   "Quote special characters for incremental search."
@@ -1108,7 +1113,6 @@ If you want to search for just a space, type C-q SPC."
   (isearch-search-and-update))
 
 
-;;===========================================================
 ;; Search Ring
 
 (defun isearch-ring-adjust1 (advance)
@@ -1207,7 +1211,7 @@ If you want to search for just a space, type C-q SPC."
       ;; isearch-string stays the same
       t)
      ((or completion ; not nil, must be a string
-	  (= 0 (length isearch-string))) ; shouldnt have to say this
+	  (= 0 (length isearch-string))) ; shouldn't have to say this
       (if (equal completion isearch-string)  ;; no extension?
 	  (if completion-auto-help
 	      (with-output-to-temp-buffer "*Isearch completions*"
@@ -1240,7 +1244,6 @@ If there is no completion possible, say so and continue searching."
 	(insert isearch-string))))
 
 
-;;;==============================================================
 ;; The search status stack (and isearch window-local variables, not used).
 ;; Need a structure for this.
 
@@ -1274,7 +1277,6 @@ If there is no completion possible, say so and continue searching."
 	      isearch-cmds)))
 
 
-;;;==================================================================
 ;; Message string
 
 (defun isearch-message (&optional c-q-hack ellipsis)
@@ -1285,7 +1287,10 @@ If there is no completion possible, say so and continue searching."
 	    isearch-message
 	    (isearch-message-suffix c-q-hack ellipsis)
 	    )))
-    (if c-q-hack m (message "%s" m))))
+    (if c-q-hack
+	m
+      (let ((message-log-max nil))
+	(message "%s" m)))))
 
 (defun isearch-message-prefix (&optional c-q-hack ellipsis nonincremental)
   ;; If about to search, and previous search regexp was invalid,
@@ -1300,6 +1305,11 @@ If there is no completion possible, say so and continue searching."
   ;; If currently failing, display no ellipsis.
   (or isearch-success (setq ellipsis nil))
   (let ((m (concat (if isearch-success "" "failing ")
+		   (if (and isearch-wrapped
+			    (if isearch-forward
+				(> (point) isearch-opoint)
+			      (< (point) isearch-opoint)))
+		       "over")
 		   (if isearch-wrapped "wrapped ")
 		   (if isearch-word "word " "")
 		   (if isearch-regexp "regexp " "")
@@ -1317,7 +1327,6 @@ If there is no completion possible, say so and continue searching."
 	    "")))
 
 
-;;;========================================================
 ;;; Searching
 
 (defun isearch-search ()
@@ -1342,6 +1351,7 @@ If there is no completion possible, say so and continue searching."
 		     (t
 		      (if isearch-forward 'search-forward 'search-backward)))
 	       isearch-string nil t))
+	(setq isearch-just-started nil)
 	(if isearch-success
 	    (setq isearch-other-end
 		  (if isearch-forward (match-beginning 0) (match-end 0)))))
@@ -1370,7 +1380,6 @@ If there is no completion possible, say so and continue searching."
 
 
 
-;;;========================================================
 ;;; Highlighting
 
 (defvar isearch-overlay nil)
@@ -1388,13 +1397,12 @@ If there is no completion possible, say so and continue searching."
   (if isearch-overlay
       (delete-overlay isearch-overlay)))
 
-;;;===========================================================
 ;;; General utilities
 
 
 (defun isearch-no-upper-case-p (string regexp-flag)
   "Return t if there are no upper case chars in STRING.
-If REGEXP-FLAG is non-nil, disregard letters preceeded by `\\' (but not `\\\\')
+If REGEXP-FLAG is non-nil, disregard letters preceded by `\\' (but not `\\\\')
 since they have special meaning in a regexp."
   (let ((case-fold-search nil))
     (not (string-match (if regexp-flag "\\(^\\|\\\\\\\\\\|[^\\]\\)[A-Z]"
@@ -1402,7 +1410,6 @@ since they have special meaning in a regexp."
 		       string))))
 
 
-;;;=================================================
 ;; Portability functions to support various Emacs versions.
 
 ;; To quiet the byte-compiler.
@@ -1411,36 +1418,22 @@ since they have special meaning in a regexp."
 (defvar last-command-event)
 
 (defun isearch-char-to-string (c)
-  (if (integerp c)
-      (make-string 1 c)
-    (if (and (symbolp c) (get c 'ascii-character))
-	(make-string 1 (get c 'ascii-character))
-      (make-string 1 (event-to-character c)))))
+  (make-string 1 c))
 
 (defun isearch-text-char-description (c)
   (if (and (integerp c) (or (< c ?\ ) (= c ?\^?)))
       (text-char-description c)
     (isearch-char-to-string c)))
 
+;; General function to unread characters or events.
+;; Also insert them in a keyboard macro being defined.
 (defun isearch-unread (&rest char-or-events)
-  ;; General function to unread characters or events.
-  (if isearch-gnu-emacs-events
-      (setq unread-command-events
-	    (append char-or-events unread-command-events))
-    (let ((char (if (cdr char-or-events)
-		    (progn
-		      (while (cdr char-or-events)
-			(setq char-or-events (cdr char-or-events)))
-		      (+ 128 (car char-or-events)))
-		  (car char-or-events))))
-      (if isearch-event-data-type
-	  (setq unread-command-event char)
-	(setq unread-command-char char)))))
+  (mapcar 'store-kbd-macro-event char-or-events)
+  (setq unread-command-events
+	(append char-or-events unread-command-events)))
 
 (defun isearch-last-command-char ()
   ;; General function to return the last command character.
-  (if isearch-event-data-type
-      last-command-event
-    last-command-char))
+  last-command-char)
 
 ;;; isearch.el ends here
