@@ -15,7 +15,8 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Emacs; see the file COPYING.  If not, write to
-the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
+the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+Boston, MA 02111-1307, USA.  */
 
 #ifdef MSDOS
 #include <dpmi.h>
@@ -29,8 +30,9 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #include <sys/resource.h>
 #define BSD4_2			/* Tell code below to use getrlimit.  */
 
-extern int __data_start;
-#define start_of_data()	&__data_start
+/* Old Linux startup code won't define __data_start.  */
+extern int etext, __data_start; weak_symbol (__data_start)
+#define start_of_data()	(&__data_start ?: &etext)
 
 #else /* not _LIBC */
 
@@ -88,7 +90,7 @@ extern char etext;
 #endif
 #endif
 
-#else  /* not emacs */ 
+#else  /* not emacs */
 extern char etext;
 #define start_of_data() &etext
 #endif /* not emacs */
